@@ -9,6 +9,7 @@ import re
 import rich
 import getpass
 import json
+import re
 from time import sleep
 from threading import Thread as thr
 from functools import wraps
@@ -210,6 +211,47 @@ class Main(cmd.Cmd):
         self.code_theme = "monokai"
         self.quiet = quiet
 
+    def output_bond(
+        self,
+        title: str,
+        text: str,
+        color: str = "cyan",
+        frame: bool = True,
+        is_json: bool = False,
+    ):
+        """Print prettified output
+
+        Args:
+            title (str): Title
+            text (str): Info to be printed
+            color (str, optional): Output color. Defaults to "cyan".
+            frame (bool, optional): Add frame. Defaults to True.
+        """
+        if is_json:
+            text = f"""
+```json
+{json.dumps(text,indent=4)}
+```
+"""
+        rich.print(
+            Panel(
+                Markdown(text, code_theme=self.code_theme),
+                title=title.title(),
+                style=Style(
+                    color=color,
+                    frame=frame,
+                ),
+            ),
+        )
+        if is_json and click.confirm("Do you wish to save this"):
+            default_path = title + ".json"
+            save_to = click.prompt(
+                "Enter path to save to", default=default_path, type=click.STRING
+            )
+            with open(save_to, "a") as fh:
+                json.dump(text, fh, indent=4)
+            click.secho(f"Successfuly saved to `{save_to}`", fg="green")
+
     @busy_bar.run("Settings saved")
     def do_settings(self, line):
         """Configre settings"""
@@ -349,6 +391,32 @@ class Main(cmd.Cmd):
         sys.stdout.write("\u001b[2J\u001b[H")
         sys.stdout.flush()
 
+    @busy_bar.run("While handling history")
+    def do_history(self, line):
+        """Show current conversation history"""
+        history = self.bot.conversation.chat_history
+        history = re.sub(
+            "\nLLM :",
+            "\n\n**LLM** :",
+            re.sub("\nUser :", "\n\n**User** :", history),
+        )
+        self.output_bond("Chat History", history, self.color)
+        if click.confirm("Do you wish to save this chat"):
+            save_to = click.prompt(
+                "Enter path/file-name", default="llama-conversation.txt"
+            )
+            with open(save_to, "a") as fh:
+                fh.write(history)
+            click.secho(f"Conversation saved successfully to '{save_to}'", fg="cyan")
+
+    @busy_bar.run("while resetting conversation")
+    def do_reset(self, line):
+        """Start new conversation thread"""
+        self.bot.conversation.chat_history = click.prompt(
+            "Introductory prompt", default=self.bot.conversation.intro
+        )
+        click.secho("Conversation reset successfully. New one created.", fg="cyan")
+
     @busy_bar.run()
     def default(self, line):
         """Chat with ChatGPT"""
@@ -533,6 +601,27 @@ def interactive(
     quiet,
 ):
     """Chat with AI interactively"""
+    if not 1:  # I tried this but it failed; just continuously raising EOFError.
+        # Consider giving it a fix if possible
+        # Let's try to read piped input
+        import signal
+
+        def timeout_handler(signum, frame):
+            raise TimeoutError("Timed out while waiting for input")
+
+        signal.signal(signal.SIGALRM, timeout_handler)
+        signal.alarm(
+            1
+        )  # Not to wait for long. I was thinking of making it 2 but I found it awful, what's your take?
+        try:
+            piped_input = click.get_text_stream("stdin").read()
+        except (TimeoutError, EOFError) as e:
+            pass
+        else:
+            prompt = piped_input
+        finally:
+            signal.alarm(0)  # Reset the alarm if input is read successfully
+            # Just incase the previous timeout was not 0
     bot = Main(
         max_tokens,
         temperature,
@@ -618,7 +707,7 @@ def interactive(
 @click.option(
     "-to", "--timeout", help="Http requesting timeout", type=click.INT, default=30
 )
-@click.argument("prompt", required=True)
+@click.argument("prompt", required=False)
 @click.option(
     "--prettify/--raw", help="Flag for prettifying markdowned response", default=True
 )
@@ -719,6 +808,33 @@ def generate(
         proxy_path,
         quiet,
     )
+    if not prompt:
+        # Let's try to read piped input
+        import signal
+
+        def timeout_handler(signum, frame):
+            raise TimeoutError("Timed out while waiting for input")
+
+        signal.signal(signal.SIGALRM, timeout_handler)
+        signal.alarm(
+            1
+        )  # Not to wait for long. I was thinking of making it 2 but I found it awful, what's your take?
+        try:
+            prompt = click.get_text_stream("stdin").read()
+        except TimeoutError as e:
+            help_info = (
+                "Usage: tgpt generate [OPTIONS] PROMPT\n"
+                "Try 'tgpt generate --help' for help.\n"
+                "Error: Missing argument 'PROMPT'.\n"
+            )
+            click.secho(
+                help_info
+            )  # Let's try to mimic the click's missing argument help info
+            exit(1)
+        else:
+            signal.alarm(0)  # Reset the alarm if input is read successfully
+            # Just incase the previous timeout was not 0
+
     prompt = Optimizers.code(prompt) if code else prompt
     prompt = Optimizers.shell_command(prompt) if shell else prompt
     busy_bar.spin_index = busy_bar_index
